@@ -61,6 +61,7 @@ struct _CcBackgroundPanelPrivate
   CcBackgroundItem *current_background;
 
   GCancellable *copy_cancellable;
+  GCancellable *capture_cancellable;
 
   GtkWidget *spinner;
 
@@ -97,6 +98,15 @@ cc_background_panel_dispose (GObject *object)
       priv->copy_cancellable = NULL;
     }
 
+  if (priv->capture_cancellable)
+    {
+      /* cancel screenshot operations */
+      g_cancellable_cancel (priv->capture_cancellable);
+
+      g_object_unref (priv->capture_cancellable);
+      priv->capture_cancellable = NULL;
+    }
+
   g_clear_object (&priv->thumb_factory);
   g_clear_object (&priv->display_screenshot);
 
@@ -242,6 +252,10 @@ on_screenshot_finished (GObject *source,
                                  &error);
 
   if (error != NULL) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+      g_error_free (error);
+      return;
+    }
     g_debug ("Unable to get screenshot: %s",
              error->message);
     g_error_free (error);
@@ -332,7 +346,7 @@ get_screenshot_async (CcBackgroundPanel *panel,
                           NULL,
                           G_DBUS_CALL_FLAGS_NONE,
                           -1,
-                          NULL,
+                          panel->priv->capture_cancellable,
                           on_screenshot_finished,
                           panel);
 }
@@ -455,9 +469,11 @@ copy_finished_cb (GObject      *source_object,
 
   if (!g_file_copy_finish (G_FILE (source_object), result, &err))
     {
-      if (err->code != G_IO_ERROR_CANCELLED)
-        g_warning ("Failed to copy image to cache location: %s", err->message);
-
+      if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+        g_error_free (err);
+        return;
+      }
+      g_warning ("Failed to copy image to cache location: %s", err->message);
       g_error_free (err);
     }
   item = g_object_get_data (source_object, "item");
@@ -707,6 +723,7 @@ cc_background_panel_init (CcBackgroundPanel *self)
                     self);
 
   priv->copy_cancellable = g_cancellable_new ();
+  priv->capture_cancellable = g_cancellable_new ();
 
   priv->thumb_factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
 
