@@ -54,6 +54,7 @@ struct CcBluetoothPanelPrivate {
 	BluetoothClient     *client;
 	BluetoothKillswitch *killswitch;
 	gboolean             debug;
+	GHashTable          *connecting_devices;
 };
 
 static void cc_bluetooth_panel_finalize (GObject *object);
@@ -96,24 +97,56 @@ cc_bluetooth_panel_finalize (GObject *object)
 	bluetooth_plugin_manager_cleanup ();
 
 	self = CC_BLUETOOTH_PANEL (object);
-	if (self->priv->builder) {
-		g_object_unref (self->priv->builder);
-		self->priv->builder = NULL;
-	}
-	if (self->priv->killswitch) {
-		g_object_unref (self->priv->killswitch);
-		self->priv->killswitch = NULL;
-	}
-	if (self->priv->client) {
-		g_object_unref (self->priv->client);
-		self->priv->client = NULL;
-	}
+	g_clear_object (&self->priv->builder);
+	g_clear_object (&self->priv->killswitch);
+	g_clear_object (&self->priv->client);
 
+	g_clear_pointer (&self->priv->connecting_devices, g_hash_table_destroy);
 	g_clear_pointer (&self->priv->selected_bdaddr, g_free);
 
 	G_OBJECT_CLASS (cc_bluetooth_panel_parent_class)->finalize (object);
 }
 
+enum {
+	CONNECTING_NOTEBOOK_PAGE_SWITCH = 0,
+	CONNECTING_NOTEBOOK_PAGE_SPINNER = 1
+};
+
+static void
+set_connecting_page (CcBluetoothPanel *self,
+		     int               page)
+{
+	if (page == CONNECTING_NOTEBOOK_PAGE_SPINNER)
+		gtk_spinner_start (GTK_SPINNER (WID ("connecting_spinner")));
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (WID ("connecting_notebook")), page);
+	if (page == CONNECTING_NOTEBOOK_PAGE_SWITCH)
+		gtk_spinner_start (GTK_SPINNER (WID ("connecting_spinner")));
+}
+
+static void
+remove_connecting (CcBluetoothPanel *self,
+		   const char       *bdaddr)
+{
+	g_hash_table_remove (self->priv->connecting_devices, bdaddr);
+}
+
+static void
+add_connecting (CcBluetoothPanel *self,
+		const char       *bdaddr)
+{
+	g_hash_table_insert (self->priv->connecting_devices,
+			     g_strdup (bdaddr),
+			     GINT_TO_POINTER (1));
+}
+
+static gboolean
+is_connecting (CcBluetoothPanel *self,
+	       const char       *bdaddr)
+{
+	return GPOINTER_TO_INT (g_hash_table_lookup (self->priv->connecting_devices,
+						     bdaddr));
+}
+
 typedef struct {
 	char             *bdaddr;
 	CcBluetoothPanel *self;
@@ -134,7 +167,7 @@ connect_done (GObject      *source_object,
 
 	self = data->self;
 
-	/* Check whether the same device is now selected */
+	/* Check whether the same device is now selected, and update the UI */
 	bdaddr = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (self->priv->chooser));
 	if (g_strcmp0 (bdaddr, data->bdaddr) == 0) {
 		GtkSwitch *button;
@@ -143,9 +176,11 @@ connect_done (GObject      *source_object,
 		/* Reset the switch if it failed */
 		if (success == FALSE)
 			gtk_switch_set_active (button, !gtk_switch_get_active (button));
-		gtk_widget_set_sensitive (GTK_WIDGET (button), TRUE);
+		set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SWITCH);
 	}
 
+	remove_connecting (self, data->bdaddr);
+
 	g_free (bdaddr);
 	g_object_unref (data->self);
 	g_free (data->bdaddr);
@@ -160,6 +195,13 @@ switch_connected_active_changed (GtkSwitch        *button,
 	char *proxy;
 	GValue value = { 0, };
 	ConnectData *data;
+	char *bdaddr;
+
+	bdaddr = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (self->priv->chooser));
+	if (is_connecting (self, bdaddr)) {
+		g_free (bdaddr);
+		return;
+	}
 
 	if (bluetooth_chooser_get_selected_device_info (BLUETOOTH_CHOOSER (self->priv->chooser),
 							"proxy", &value) == FALSE) {
@@ -173,7 +215,7 @@ switch_connected_active_changed (GtkSwitch        *button,
 		return;
 
 	data = g_new0 (ConnectData, 1);
-	data->bdaddr = bluetooth_chooser_get_selected_device (BLUETOOTH_CHOOSER (self->priv->chooser));
+	data->bdaddr = bdaddr;
 	data->self = g_object_ref (self);
 
 	bluetooth_client_connect_service (self->priv->client,
@@ -183,9 +225,8 @@ switch_connected_active_changed (GtkSwitch        *button,
 					  connect_done,
 					  data);
 
-	/* FIXME: make a note somewhere that the button for that
-	 * device should be disabled */
-	gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
+	add_connecting (self, data->bdaddr);
+	set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SPINNER);
 	g_free (proxy);
 }
 
@@ -280,8 +321,14 @@ cc_bluetooth_panel_update_properties (CcBluetoothPanel *self)
 
 		gtk_widget_set_sensitive (WID ("properties_vbox"), TRUE);
 
-		connected = bluetooth_chooser_get_selected_device_is_connected (BLUETOOTH_CHOOSER (self->priv->chooser));
-		gtk_switch_set_active (button, connected);
+		if (is_connecting (self, bdaddr)) {
+			gtk_switch_set_active (button, TRUE);
+			set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SPINNER);
+		} else {
+			connected = bluetooth_chooser_get_selected_device_is_connected (BLUETOOTH_CHOOSER (self->priv->chooser));
+			gtk_switch_set_active (button, connected);
+			set_connecting_page (self, CONNECTING_NOTEBOOK_PAGE_SWITCH);
+		}
 
 		/* Paired */
 		bluetooth_chooser_get_selected_device_info (BLUETOOTH_CHOOSER (self->priv->chooser),
@@ -384,7 +431,6 @@ cc_bluetooth_panel_update_power (CcBluetoothPanel *self)
 	BluetoothKillswitchState state;
 	char *path;
 	gboolean powered, sensitive;
-	GtkSwitch *button;
 
 	g_object_get (G_OBJECT (self->priv->client),
 		      "default-adapter", &path,
@@ -421,12 +467,6 @@ cc_bluetooth_panel_update_power (CcBluetoothPanel *self)
 	g_free (path);
 	gtk_widget_set_sensitive (WID ("box_power") , sensitive);
 	gtk_widget_set_sensitive (WID ("box_vis") , sensitive);
-
-	button = GTK_SWITCH (WID ("switch_bluetooth"));
-
-	g_signal_handlers_block_by_func (button, power_callback, self);
-	gtk_switch_set_active (button, powered);
-	g_signal_handlers_unblock_by_func (button, power_callback, self);
 }
 
 static void
@@ -696,23 +736,43 @@ static void
 cc_bluetooth_panel_update_state (CcBluetoothPanel *self)
 {
 	char *bdaddr;
-	gboolean powered;
 
 	g_object_get (G_OBJECT (self->priv->client),
 		      "default-adapter", &bdaddr,
-		      "default-adapter-powered", &powered,
 		      NULL);
 	gtk_widget_set_sensitive (WID ("toolbar"), (bdaddr != NULL));
 	g_free (bdaddr);
 }
 
+static void
+cc_bluetooth_panel_update_powered_state (CcBluetoothPanel *self)
+{
+	gboolean powered;
+
+	g_object_get (G_OBJECT (self->priv->client),
+		      "default-adapter-powered", &powered,
+		      NULL);
+	gtk_switch_set_active (GTK_SWITCH (WID ("switch_bluetooth")), powered);
+}
+
+static void
+default_adapter_power_changed (BluetoothClient  *client,
+			       GParamSpec       *spec,
+			       CcBluetoothPanel *self)
+{
+	g_debug ("Default adapter power changed");
+	cc_bluetooth_panel_update_powered_state (self);
+}
+
 static void
 default_adapter_changed (BluetoothClient  *client,
 			 GParamSpec       *spec,
 			 CcBluetoothPanel *self)
 {
+	g_debug ("Default adapter changed");
 	cc_bluetooth_panel_update_state (self);
 	cc_bluetooth_panel_update_power (self);
+	cc_bluetooth_panel_update_powered_state (self);
 }
 
 static void
@@ -720,6 +780,7 @@ killswitch_changed (BluetoothKillswitch      *killswitch,
 		    BluetoothKillswitchState  state,
 		    CcBluetoothPanel         *self)
 {
+	g_debug ("Killswitch changed to state '%s' (%d)", bluetooth_killswitch_state_to_string (state) , state);
 	cc_bluetooth_panel_update_state (self);
 	cc_bluetooth_panel_update_power (self);
 }
@@ -736,6 +797,10 @@ cc_bluetooth_panel_init (CcBluetoothPanel *self)
 	bluetooth_plugin_manager_init ();
 	self->priv->killswitch = bluetooth_killswitch_new ();
 	self->priv->client = bluetooth_client_new ();
+	self->priv->connecting_devices = g_hash_table_new_full (g_str_hash,
+								g_str_equal,
+								(GDestroyNotify) g_free,
+								NULL);
 	self->priv->debug = g_getenv ("BLUETOOTH_DEBUG") != NULL;
 
 	self->priv->builder = gtk_builder_new ();
@@ -756,6 +821,8 @@ cc_bluetooth_panel_init (CcBluetoothPanel *self)
 	cc_bluetooth_panel_update_state (self);
 	g_signal_connect (G_OBJECT (self->priv->client), "notify::default-adapter",
 			  G_CALLBACK (default_adapter_changed), self);
+	g_signal_connect (G_OBJECT (self->priv->client), "notify::default-adapter-powered",
+			  G_CALLBACK (default_adapter_power_changed), self);
 
 	/* The discoverable button */
 	cc_bluetooth_panel_update_visibility (self);
