@@ -1,4 +1,8 @@
+#include "config.h"
+
 #include <glib.h>
+#include <glib/gi18n.h>
+#include <locale.h>
 #include <glib-object.h>
 #include "gdm-languages.h"
 
@@ -7,6 +11,9 @@ int main (int argc, char **argv)
 	char **langs;
 	guint i;
 
+	setlocale (LC_ALL, NULL);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
 	g_type_init ();
 
 	if (argc > 1) {
