@@ -37,7 +37,7 @@
 #include <glib/gi18n.h>
 #include <gdesktop-enums.h>
 
-G_DEFINE_DYNAMIC_TYPE (CcDisplayPanel, cc_display_panel, CC_TYPE_PANEL)
+CC_PANEL_REGISTER (CcDisplayPanel, cc_display_panel)
 
 #define DISPLAY_PANEL_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), CC_TYPE_DISPLAY_PANEL, CcDisplayPanelPrivate))
@@ -169,9 +169,13 @@ cc_display_panel_finalize (GObject *object)
     g_object_unref (self->priv->clock_settings);
 
   shell = cc_panel_get_shell (CC_PANEL (self));
-  toplevel = cc_shell_get_toplevel (shell);
-  g_signal_handler_disconnect (G_OBJECT (toplevel),
-                               self->priv->focus_id);
+  if (shell != NULL)
+    {
+      toplevel = cc_shell_get_toplevel (shell);
+      if (toplevel != NULL)
+        g_signal_handler_disconnect (G_OBJECT (toplevel),
+                                     self->priv->focus_id);
+    }
 
   gnome_rr_labeler_hide (self->priv->labeler);
   g_object_unref (self->priv->labeler);
@@ -202,11 +206,6 @@ cc_display_panel_class_init (CcDisplayPanelClass *klass)
   object_class->finalize = cc_display_panel_finalize;
 }
 
-static void
-cc_display_panel_class_finalize (CcDisplayPanelClass *klass)
-{
-}
-
 static void
 error_message (CcDisplayPanel *self, const char *primary_text, const char *secondary_text)
 {
@@ -614,12 +613,14 @@ rebuild_mirror_screens (CcDisplayPanel *self)
 static char *
 mirror_monitor_name (void)
 {
+  /* Keep this string in sync with gnome-desktop/libgnome-desktop/gnome-rr-labeler.c */
+
   /* Translators:  this is the feature where what you see on your laptop's
-   * screen is the same as your external monitor.  Here, "Mirror" is being
-   * used as an adjective, not as a verb.  For example, the Spanish
-   * translation could be "Pantallas en Espejo", *not* "Espejar Pantallas".
+   * screen is the same as your external projector.  Here, "Mirrored" is being
+   * used as an adjective.  For example, the Spanish translation could be
+   * "Pantallas en Espejo".
    */
-  return g_strdup (_("Mirror Displays"));
+  return g_strdup (_("Mirrored Displays"));
 }
 
 static void
