@@ -38,6 +38,8 @@
 #include "net-proxy.h"
 #include "net-vpn.h"
 
+#include "rfkill-glib.h"
+
 #include "panel-common.h"
 
 #include "network-dialogs.h"
@@ -67,7 +69,12 @@ struct _CcNetworkPanelPrivate
         guint             add_header_widgets_idle;
         guint             nm_warning_idle;
         guint             refresh_idle;
+
+        /* Killswitch stuff */
         GtkWidget        *kill_switch_header;
+        CcRfkillGlib       *rfkill;
+        GtkSwitch        *rfkill_switch;
+        GHashTable       *killswitches;
 
         /* wireless dialog stuff */
         CmdlineOperation  arg_operation;
@@ -202,6 +209,9 @@ cc_network_panel_dispose (GObject *object)
         g_clear_object (&priv->client);
         g_clear_object (&priv->remote_settings);
         g_clear_object (&priv->kill_switch_header);
+        g_clear_object (&priv->rfkill);
+        g_clear_pointer (&priv->killswitches, g_hash_table_destroy);
+        priv->rfkill_switch = NULL;
 
         if (priv->refresh_idle != 0) {
                 g_source_remove (priv->refresh_idle);
@@ -333,55 +343,6 @@ object_removed_cb (NetObject *object, CcNetworkPanel *panel)
         } while (gtk_tree_model_iter_next (model, &iter));
 }
 
-static void
-panel_refresh_killswitch_visibility (CcNetworkPanel *panel)
-{
-        gboolean ret;
-        gboolean show_flight_toggle = FALSE;
-        GtkTreeIter iter;
-        GtkTreeModel *model;
-        NetObject *object_tmp;
-        NMDeviceModemCapabilities caps;
-        NMDevice *nm_device;
-
-        /* find any wireless devices in model */
-        model = GTK_TREE_MODEL (gtk_builder_get_object (panel->priv->builder,
-                                                        "liststore_devices"));
-        ret = gtk_tree_model_get_iter_first (model, &iter);
-        if (!ret)
-                return;
-        do {
-                gtk_tree_model_get (model, &iter,
-                                    PANEL_DEVICES_COLUMN_OBJECT, &object_tmp,
-                                    -1);
-                if (NET_IS_DEVICE (object_tmp)) {
-                        nm_device = net_device_get_nm_device (NET_DEVICE (object_tmp));
-                        switch (nm_device_get_device_type (nm_device)) {
-                        case NM_DEVICE_TYPE_WIFI:
-                        case NM_DEVICE_TYPE_WIMAX:
-                                show_flight_toggle = TRUE;
-                                break;
-                        case NM_DEVICE_TYPE_MODEM:
-                                {
-                                caps = nm_device_modem_get_current_capabilities (NM_DEVICE_MODEM (nm_device));
-                                if ((caps & NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS) ||
-                                    (caps & NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO))
-                                        show_flight_toggle = TRUE;
-                                }
-                                break;
-                        default:
-                                break;
-                        }
-                }
-                if (object_tmp != NULL)
-                        g_object_unref (object_tmp);
-        } while (!show_flight_toggle && gtk_tree_model_iter_next (model, &iter));
-
-        /* only show toggle if there are wireless devices */
-        gtk_widget_set_visible (panel->priv->kill_switch_header,
-                                show_flight_toggle);
-}
-
 static gboolean
 handle_argv_for_device (CcNetworkPanel *panel,
 			NMDevice       *device,
@@ -700,11 +661,18 @@ cc_network_panel_notify_enable_active_cb (GtkSwitch *sw,
                                           GParamSpec *pspec,
                                           CcNetworkPanel *panel)
 {
-        gboolean enable;
+	gboolean enable;
+	struct rfkill_event event;
 
-        /* set enabled state */
-        enable = !gtk_switch_get_active (sw);
-        nm_client_wireless_set_enabled (panel->priv->client, enable);
+	enable = gtk_switch_get_active (sw);
+	g_debug ("Setting killswitch to %d", enable);
+
+	memset (&event, 0, sizeof(event));
+	event.op = RFKILL_OP_CHANGE_ALL;
+	event.type = RFKILL_TYPE_ALL;
+	event.soft = enable ? 1 : 0;
+	if (cc_rfkill_glib_send_event (panel->priv->rfkill, &event) < 0)
+		g_warning ("Setting the killswitch %s failed", enable ? "on" : "off");
 }
 
 static void
@@ -746,7 +714,6 @@ device_added_cb (NMClient *client, NMDevice *device, CcNetworkPanel *panel)
 {
         g_debug ("New device added");
         panel_add_device (panel, device);
-        panel_refresh_killswitch_visibility (panel);
 }
 
 static void
@@ -754,7 +721,6 @@ device_removed_cb (NMClient *client, NMDevice *device, CcNetworkPanel *panel)
 {
         g_debug ("Device removed");
         panel_remove_device (panel, device);
-        panel_refresh_killswitch_visibility (panel);
 }
 
 static void
@@ -1059,11 +1025,64 @@ on_toplevel_map (GtkWidget      *widget,
         }
 }
 
+static void
+rfkill_changed (CcRfkillGlib     *rfkill,
+		GList          *events,
+		CcNetworkPanel *panel)
+{
+	gboolean enabled;
+	GList *l;
+	GHashTableIter iter;
+	gpointer key, value;
+
+	enabled = TRUE;
+
+	for (l = events; l != NULL; l = l->next) {
+		struct rfkill_event *event = l->data;
+
+		if (event->op == RFKILL_OP_ADD)
+			g_hash_table_insert (panel->priv->killswitches,
+					     GINT_TO_POINTER (event->idx),
+					     GINT_TO_POINTER (event->soft || event->hard));
+		else if (event->op == RFKILL_OP_CHANGE)
+			g_hash_table_insert (panel->priv->killswitches,
+					     GINT_TO_POINTER (event->idx),
+					     GINT_TO_POINTER (event->soft || event->hard));
+		else if (event->op == RFKILL_OP_DEL)
+			g_hash_table_remove (panel->priv->killswitches,
+					     GINT_TO_POINTER (event->idx));
+	}
+
+	g_hash_table_iter_init (&iter, panel->priv->killswitches);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		int idx, state;
+
+		idx = GPOINTER_TO_INT (key);
+		state = GPOINTER_TO_INT (value);
+		g_debug ("Killswitch %d is %s", idx, state ? "enabled" : "disabled");
+
+		/* A single device that's enabled? airplane mode is off */
+		if (state == FALSE) {
+			enabled = FALSE;
+			break;
+		}
+	}
+
+	if (enabled != gtk_switch_get_active (panel->priv->rfkill_switch)) {
+		g_signal_handlers_block_by_func (panel->priv->rfkill_switch,
+						 cc_network_panel_notify_enable_active_cb,
+						 panel);
+		gtk_switch_set_active (panel->priv->rfkill_switch, enabled);
+		g_signal_handlers_unblock_by_func (panel->priv->rfkill_switch,
+						 cc_network_panel_notify_enable_active_cb,
+						 panel);
+	}
+}
+
 static gboolean
 network_add_shell_header_widgets_cb (gpointer user_data)
 {
         CcNetworkPanel *panel = CC_NETWORK_PANEL (user_data);
-        gboolean ret;
         GtkWidget *box;
         GtkWidget *label;
         GtkWidget *widget;
@@ -1077,16 +1096,21 @@ network_add_shell_header_widgets_cb (gpointer user_data)
         widget = gtk_switch_new ();
         gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
         gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 0);
-        gtk_widget_set_visible (widget, TRUE);
+        gtk_widget_show_all (box);
+        panel->priv->rfkill_switch = GTK_SWITCH (widget);
         cc_shell_embed_widget_in_header (cc_panel_get_shell (CC_PANEL (panel)), box);
         panel->priv->kill_switch_header = g_object_ref (box);
 
-        ret = nm_client_wireless_get_enabled (panel->priv->client);
-        gtk_switch_set_active (GTK_SWITCH (widget), !ret);
-        g_signal_connect (GTK_SWITCH (widget), "notify::active",
+        panel->priv->killswitches = g_hash_table_new (g_direct_hash, g_direct_equal);
+        panel->priv->rfkill = cc_rfkill_glib_new ();
+        g_signal_connect (G_OBJECT (panel->priv->rfkill), "changed",
+                          G_CALLBACK (rfkill_changed), panel);
+        if (cc_rfkill_glib_open (panel->priv->rfkill) < 0)
+                gtk_widget_hide (box);
+
+        g_signal_connect (panel->priv->rfkill_switch, "notify::active",
                           G_CALLBACK (cc_network_panel_notify_enable_active_cb),
                           panel);
-        panel_refresh_killswitch_visibility (panel);
 
         return FALSE;
 }
