@@ -175,39 +175,6 @@ add_access_point (NetDeviceWifi *device_wifi, NMAccessPoint *ap, NMAccessPoint *
         g_free (title);
 }
 
-static void
-add_connect_to_hidden (NetDeviceWifi *device_wifi)
-{
-        NetDeviceWifiPrivate *priv = device_wifi->priv;
-        GtkListStore *liststore_network;
-        GtkTreeIter treeiter;
-        gchar *title;
-
-        liststore_network = GTK_LIST_STORE (gtk_builder_get_object (priv->builder,
-                                                     "liststore_network"));
-
-        /* TRANSLATORS: this is when the access point is not shown
-         * in the list and the user has to enter the SSID manually
-         */
-        title = g_strdup_printf ("<b>%s</b>", _("Connect to a Hidden Network"));
-        gtk_list_store_append (liststore_network, &treeiter);
-        gtk_list_store_set (liststore_network,
-                            &treeiter,
-                            COLUMN_CONNECTION_ID, "ap-other...",
-                            COLUMN_ACCESS_POINT_ID, "ap-other...",
-                            COLUMN_TITLE, title,
-                            /* always last */
-                            COLUMN_SORT, "",
-                            COLUMN_STRENGTH, 0,
-                            COLUMN_MODE, NM_802_11_MODE_UNKNOWN,
-                            COLUMN_SECURITY, NM_AP_SEC_UNKNOWN,
-                            COLUMN_AP_IN_RANGE, FALSE,
-                            COLUMN_AP_OUT_OF_RANGE, FALSE,
-                            COLUMN_AP_IS_SAVED, FALSE,
-                            -1);
-        g_free (title);
-}
-
 static GPtrArray *
 panel_get_strongest_unique_aps (const GPtrArray *aps)
 {
@@ -518,7 +485,6 @@ device_wifi_refresh_aps (NetDeviceWifi *device_wifi)
                 ap = NM_ACCESS_POINT (g_ptr_array_index (aps_unique, i));
                 add_access_point (device_wifi, ap, active_ap, nm_device);
         }
-        add_connect_to_hidden (device_wifi);
 
         device_wifi->priv->updating_device = FALSE;
         g_ptr_array_unref (aps_unique);
@@ -1037,6 +1003,9 @@ connection_add_activate_cb (NMClient *client,
 
         if (connection == NULL) {
                 /* failed to activate */
+                g_debug ("Failed to add and activate connection '%d': %s",
+                         error->code,
+                         error->message);
                 nm_device_wifi_refresh_ui (device_wifi);
         }
 }
@@ -1051,6 +1020,9 @@ connection_activate_cb (NMClient *client,
 
         if (connection == NULL) {
                 /* failed to activate */
+                g_debug ("Failed to activate connection '%d': %s",
+                         error->code,
+                         error->message);
                 nm_device_wifi_refresh_ui (device_wifi);
         }
 }
@@ -1074,6 +1046,28 @@ activate_connection (NetDeviceWifi *device_wifi,
                                        connection_activate_cb, device_wifi);
 }
 
+static gboolean
+is_8021x (NMDevice   *device,
+          const char *ap_object_path)
+{
+        NM80211ApSecurityFlags wpa_flags, rsn_flags;
+        NMAccessPoint *ap;
+
+        ap = nm_device_wifi_get_access_point_by_path (NM_DEVICE_WIFI (device),
+                                                      ap_object_path);
+        if (!ap)
+                return FALSE;
+
+        rsn_flags = nm_access_point_get_rsn_flags (ap);
+        if (rsn_flags & NM_802_11_AP_SEC_KEY_MGMT_802_1X)
+                return TRUE;
+
+        wpa_flags = nm_access_point_get_wpa_flags (ap);
+        if (wpa_flags & NM_802_11_AP_SEC_KEY_MGMT_802_1X)
+                return TRUE;
+        return FALSE;
+}
+
 static void
 wireless_try_to_connect (NetDeviceWifi *device_wifi,
                          const gchar *ssid_target,
@@ -1083,7 +1077,6 @@ wireless_try_to_connect (NetDeviceWifi *device_wifi,
         const gchar *ssid_tmp;
         GSList *list, *l;
         GSList *filtered;
-        NMConnection *connection;
         NMConnection *connection_activate = NULL;
         NMDevice *device;
         NMSettingWireless *setting_wireless;
@@ -1110,6 +1103,8 @@ wireless_try_to_connect (NetDeviceWifi *device_wifi,
         filtered = nm_device_filter_connections (device, list);
         g_debug ("%i suitable remote connections to check", g_slist_length (filtered));
         for (l = filtered; l; l = g_slist_next (l)) {
+                NMConnection *connection;
+
                 connection = NM_CONNECTION (l->data);
                 setting_wireless = nm_connection_get_setting_wireless (connection);
                 if (!NM_IS_SETTING_WIRELESS (setting_wireless))
@@ -1140,12 +1135,30 @@ wireless_try_to_connect (NetDeviceWifi *device_wifi,
         }
 
         /* create one, as it's missing */
-        g_debug ("no existing connection found for %s, creating",
-                 ssid_target);
-        nm_client_add_and_activate_connection (client,
-                                               NULL,
-                                               device, ap_object_path,
-                                               connection_add_activate_cb, device_wifi);
+        g_debug ("no existing connection found for %s, creating", ssid_target);
+
+        if (!is_8021x (device, ap_object_path)) {
+                g_debug ("no existing connection found for %s, creating and activating one", ssid_target);
+                nm_client_add_and_activate_connection (client,
+                                                       NULL,
+                                                       device, ap_object_path,
+                                                       connection_add_activate_cb, device_wifi);
+        } else {
+                CcNetworkPanel *panel;
+                GPtrArray *array;
+
+                g_debug ("no existing connection found for %s, creating", ssid_target);
+                array = g_ptr_array_new ();
+                g_ptr_array_add (array, "connect-8021x-wifi");
+                g_ptr_array_add (array, (gpointer) nm_object_get_path (NM_OBJECT (device)));
+                g_ptr_array_add (array, (gpointer) ap_object_path);
+                g_ptr_array_add (array, NULL);
+
+                panel = net_object_get_panel (NET_OBJECT (device_wifi));
+                g_object_set (G_OBJECT (panel), "argv", array->pdata, NULL);
+
+                g_ptr_array_free (array, FALSE);
+        }
 out:
         return;
 }
@@ -1645,15 +1658,14 @@ connect_wifi_network (NetDeviceWifi *device_wifi,
                             COLUMN_AP_IN_RANGE, &ap_in_range,
                             COLUMN_MODE, &mode,
                             -1);
-
-        if (g_strcmp0 (connection_id, "ap-other...") == 0)
-                connect_to_hidden_network (device_wifi);
-        else if (connection_id)
-                activate_connection (device_wifi, connection_id);
-        else if (ap_in_range || mode == NM_802_11_MODE_UNKNOWN)
-                wireless_try_to_connect (device_wifi, ssid, ap_object_path);
-        else
+        if (ap_in_range) {
+                if (connection_id)
+                        activate_connection (device_wifi, connection_id);
+                else
+                        wireless_try_to_connect (device_wifi, ssid, ap_object_path);
+        } else {
                 g_warning ("can't connect");
+        }
 
         g_free (ap_object_path);
         g_free (connection_id);
@@ -1739,12 +1751,23 @@ set_arrow_image (GtkCellLayout   *layout,
                  GtkTreeIter     *iter,
                  gpointer         user_data)
 {
+        NetDeviceWifi *device = user_data;
         const gchar *icon;
 
-        if (arrow_visible (model, iter))
-                icon = "go-next";
-        else
+        if (arrow_visible (model, iter)) {
+                GtkWidget *widget;
+
+                widget = GTK_WIDGET (gtk_builder_get_object (device->priv->builder,
+                                                             "treeview_list"));
+
+                if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+                        icon = "go-previous";
+                else
+                        icon = "go-next";
+        }
+        else {
                 icon = "";
+        }
 
         g_object_set (cell, "icon-name", icon, NULL);
 }
@@ -1920,7 +1943,7 @@ activate_ssid_cb (PanelCellRendererText *cell,
         GtkTreeView *tv;
         GtkTreePath *tpath;
 
-        g_print ("activate ssid!\n");
+        g_debug ("activate ssid!\n");
 
         tv = GTK_TREE_VIEW (gtk_builder_get_object (device_wifi->priv->builder,
                                                     "treeview_list"));
@@ -1941,7 +1964,7 @@ activate_arrow_cb (PanelCellRendererText *cell,
         GtkTreePath *tpath;
         GtkTreeIter iter;
 
-        g_print ("activate arrow!\n");
+        g_debug ("activate arrow!\n");
 
         tv = GTK_TREE_VIEW (gtk_builder_get_object (device_wifi->priv->builder,
                                                     "treeview_list"));
@@ -2044,6 +2067,7 @@ net_device_wifi_init (NetDeviceWifi *device_wifi)
         renderer2 = panel_cell_renderer_text_new ();
         g_object_set (renderer2,
                       "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE,
+                      "ellipsize", PANGO_ELLIPSIZE_END,
                       NULL);
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (column), renderer2, TRUE);
         gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (column), renderer2,
@@ -2149,6 +2173,12 @@ net_device_wifi_init (NetDeviceWifi *device_wifi)
                                                      "start_hotspot_button"));
         g_signal_connect (widget, "clicked",
                           G_CALLBACK (start_hotspot), device_wifi);
+
+        widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
+                                                     "connect_hidden_button"));
+        g_signal_connect_swapped (widget, "clicked",
+                                  G_CALLBACK (connect_to_hidden_network), device_wifi);
+
         widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
                                                      "switch_hotspot_off"));
         g_signal_connect (widget, "notify::active",
