@@ -57,6 +57,7 @@ enum {
 
 static GSettings *input_sources_settings = NULL;
 static GnomeXkbInfo *xkb_info = NULL;
+static GtkWidget *input_chooser = NULL; /* weak pointer */
 
 #ifdef HAVE_IBUS
 static IBusBus *ibus = NULL;
@@ -155,6 +156,7 @@ static const gchar *supported_ibus_engines[] = {
   /* Marathi */
   "m17n:mr:inscript",
   "m17n:mr:itrans",
+  "m17n:mr:phonetic",
 
   /* Nepali */
   "m17n:ne:rom",
@@ -195,6 +197,30 @@ static const gchar *supported_ibus_engines[] = {
   /* Urdu */
   "m17n:ur:phonetic",
 
+  /* Inscript2 - https://bugzilla.gnome.org/show_bug.cgi?id=684854 */
+  "m17n:as:inscript2",
+  "m17n:bn:inscript2",
+  "m17n:brx:inscript2-deva",
+  "m17n:doi:inscript2-deva",
+  "m17n:gu:inscript2",
+  "m17n:hi:inscript2",
+  "m17n:kn:inscript2",
+  "m17n:kok:inscript2-deva",
+  "m17n:mai:inscript2",
+  "m17n:ml:inscript2",
+  "m17n:mni:inscript2-beng",
+  "m17n:mni:inscript2-mtei",
+  "m17n:mr:inscript2",
+  "m17n:ne:inscript2-deva",
+  "m17n:or:inscript2",
+  "m17n:pa:inscript2-guru",
+  "m17n:sa:inscript2",
+  "m17n:sat:inscript2-deva",
+  "m17n:sat:inscript2-olck",
+  "m17n:sd:inscript2-deva",
+  "m17n:ta:inscript2",
+  "m17n:te:inscript2",
+
   /* No corresponding XKB map available for the languages */
 
   /* Chinese Yi */
@@ -253,6 +279,8 @@ static const gchar *supported_ibus_engines[] = {
 };
 #endif  /* HAVE_IBUS */
 
+static void       populate_model             (GtkListStore  *store,
+                                              GtkListStore  *active_sources_store);
 static GtkWidget *input_chooser_new          (GtkWindow     *main_window,
                                               GtkListStore  *active_sources);
 static gboolean   input_chooser_get_selected (GtkWidget     *chooser,
@@ -309,14 +337,34 @@ setup_app_info_for_id (const gchar *id)
 {
   GDesktopAppInfo *app_info;
   gchar *desktop_file_name;
+  gchar **strv;
+
+  strv = g_strsplit (id, ":", 2);
+  desktop_file_name = g_strdup_printf ("ibus-setup-%s.desktop", strv[0]);
+  g_strfreev (strv);
 
-  desktop_file_name = g_strdup_printf ("ibus-setup-%s.desktop", id);
   app_info = g_desktop_app_info_new (desktop_file_name);
   g_free (desktop_file_name);
 
   return app_info;
 }
 
+static void
+input_chooser_repopulate (GtkListStore *active_sources_store)
+{
+  GtkBuilder *builder;
+  GtkListStore *model;
+
+  if (!input_chooser)
+    return;
+
+  builder = g_object_get_data (G_OBJECT (input_chooser), "builder");
+  model = GTK_LIST_STORE (gtk_builder_get_object (builder, "input_source_model"));
+
+  gtk_list_store_clear (model);
+  populate_model (model, active_sources_store);
+}
+
 static void
 update_ibus_active_sources (GtkBuilder *builder)
 {
@@ -364,6 +412,8 @@ update_ibus_active_sources (GtkBuilder *builder)
 
       ret = gtk_tree_model_iter_next (model, &iter);
     }
+
+  input_chooser_repopulate (GTK_LIST_STORE (model));
 }
 
 static void
@@ -556,49 +606,6 @@ populate_model (GtkListStore *store,
   g_hash_table_destroy (active_sources_table);
 }
 
-GVariant *
-create_source_from_current_xkb_config (void)
-{
-  GVariantBuilder builder;
-  XkbRF_VarDefsRec *xkb_var_defs;
-  gchar *tmp;
-  gchar *source_id = NULL;
-  gchar **layouts = NULL;
-  gchar **variants = NULL;
-
-  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(ss)"));
-
-  gnome_xkb_info_get_var_defs (&tmp, &xkb_var_defs);
-  g_free (tmp);
-
-  if (xkb_var_defs->layout)
-    layouts = g_strsplit (xkb_var_defs->layout, ",", 2);
-  if (xkb_var_defs->variant)
-    variants = g_strsplit (xkb_var_defs->variant, ",", 2);
-
-  if (layouts && layouts[0] && layouts[0][0])
-    source_id = g_strdup_printf ("%s", layouts[0]);
-  else
-    goto out;
-
-  if (variants && variants[0] && variants[0][0])
-    {
-      tmp = source_id;
-      source_id = g_strdup_printf ("%s+%s", tmp, variants[0]);
-      g_free (tmp);
-    }
-
-  g_variant_builder_add (&builder, "(ss)", INPUT_SOURCE_TYPE_XKB, source_id);
-
- out:
-  g_free (source_id);
-  g_strfreev (layouts);
-  g_strfreev (variants);
-  gnome_xkb_info_free_var_defs (xkb_var_defs);
-
-  return g_variant_builder_end (&builder);
-}
-
 static void
 populate_with_active_sources (GtkListStore *store)
 {
@@ -612,11 +619,6 @@ populate_with_active_sources (GtkListStore *store)
   GtkTreeIter tree_iter;
 
   sources = g_settings_get_value (input_sources_settings, KEY_INPUT_SOURCES);
-  if (g_variant_n_children (sources) < 1)
-    {
-      g_variant_unref (sources);
-      sources = create_source_from_current_xkb_config ();
-    }
 
   g_variant_iter_init (&iter, sources);
   while (g_variant_iter_next (&iter, "(&s&s)", &type, &id))
@@ -1108,6 +1110,7 @@ show_selected_settings (GtkButton *button, gpointer data)
   GtkTreeIter iter;
   GdkAppLaunchContext *ctx;
   GDesktopAppInfo *app_info;
+  gchar *id;
   GError *error = NULL;
 
   g_debug ("show selected layout");
@@ -1123,6 +1126,12 @@ show_selected_settings (GtkButton *button, gpointer data)
   ctx = gdk_display_get_app_launch_context (gdk_display_get_default ());
   gdk_app_launch_context_set_timestamp (ctx, gtk_get_current_event_time ());
 
+  gtk_tree_model_get (model, &iter, ID_COLUMN, &id, -1);
+  g_app_launch_context_setenv (G_APP_LAUNCH_CONTEXT (ctx),
+                               "IBUS_ENGINE_NAME",
+                               id);
+  g_free (id);
+
   if (!g_app_info_launch (G_APP_INFO (app_info), NULL, G_APP_LAUNCH_CONTEXT (ctx), &error))
     {
       g_warning ("Failed to launch input source setup: %s", error->message);
@@ -1483,6 +1492,8 @@ input_chooser_new (GtkWindow    *main_window,
                              GNOMECC_UI_DIR "/gnome-region-panel-input-chooser.ui",
                              NULL);
   chooser = WID ("input_source_chooser");
+  input_chooser = chooser;
+  g_object_add_weak_pointer (G_OBJECT (chooser), (gpointer *) &input_chooser);
   g_object_set_data_full (G_OBJECT (chooser), "builder", builder, g_object_unref);
 
   filtered_list = WID ("filtered_input_source_list");
