@@ -35,7 +35,6 @@
 #include "cc-common-language.h"
 #include "gdm-languages.h"
 #include "gnome-region-panel-system.h"
-#include "gnome-region-panel-input.h"
 
 #define WID(s) GTK_WIDGET(gtk_builder_get_object (dialog, s))
 
@@ -149,11 +148,6 @@ input_sources_changed (GSettings *settings,
         const gchar *id;
 
         sources = g_settings_get_value (input_sources_settings, "sources");
-        if (g_variant_n_children (sources) < 1) {
-                g_variant_unref (sources);
-                sources = create_source_from_current_xkb_config ();
-        }
-
         xkb_info = gnome_xkb_info_new ();
 
         label = WID ("user_input_source");
@@ -319,14 +313,21 @@ on_localed_properties_changed (GDBusProxy   *proxy,
                 return;
         }
 
+        variants = NULL;
+        g_object_set_data_full (G_OBJECT (label), "input_variants", NULL, g_free);
+
         w = g_dbus_proxy_get_cached_property (proxy, "X11Variant");
         if (w) {
-                variants = g_strsplit (g_variant_get_string (w, NULL), ",", -1);
-                g_object_set_data_full (G_OBJECT (label), "input_variants",
-                                        g_variant_dup_string (w, NULL), g_free);
+                const char *variants_str;
+
+                variants_str = g_variant_get_string (w, NULL);
+                g_debug ("Got variants '%s'", variants_str);
+                if (variants_str && *variants_str != '\0') {
+                        variants = g_strsplit (variants_str, ",", -1);
+                        g_object_set_data_full (G_OBJECT (label), "input_variants",
+                                                g_strdup (variants_str), g_free);
+                }
                 g_variant_unref (w);
-        } else {
-                g_object_set_data_full (G_OBJECT (label), "input_variants", NULL, g_free);
         }
 
         if (variants && variants[0])
